/*
 * GCLC Socket, Socket implementation of GCLC
 * Copyright (C) 2014-2017 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-socket:fr.bigeon.gclc.socket.PluggableConsoleInputTest.java
 * Created on: Nov 18, 2017
 */
package fr.bigeon.gclc.socket;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

import org.junit.Test;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class PluggableConsoleInputTest {

    /** Test method for
     * {@link fr.bigeon.gclc.socket.PluggableConsoleInput#close()}. */
    @Test
    public final void testClose() {
        final PluggableConsoleInput input = new PluggableConsoleInput();
        assertFalse("Input should not be initially closed", input.isClosed());
        input.close();
        assertTrue("Close should close the input", input.isClosed());

        try {
            input.prompt();
            fail("Closed prompt should cause an IO");
        } catch (final IOException e) {
            // ok
        }
        try {
            input.prompt(10);
            fail("Closed prompt should cause an IO");
        } catch (final IOException e) {
            // ok
        }

        final PluggableConsoleInput input2 = new PluggableConsoleInput();

        final Thread th = new Thread(new Runnable() {

            @Override
            public void run() {
                while (!input2.isPrompting()) {

                }
                try {
                    Thread.sleep(100);
                } catch (final InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                input2.close();
            }
        });
        th.start();
        try {
            input.prompt();
            fail("Closed prompt should cause an IO");
        } catch (final IOException e) {
            // ok
        }
        final PluggableConsoleInput input3 = new PluggableConsoleInput();

        final Thread th2 = new Thread(new Runnable() {

            @Override
            public void run() {
                try {
                    Thread.sleep(200);
                } catch (final InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
                input3.close();
            }
        });
        th2.start();
        try {
            input.prompt(2000);
            fail("Closed prompt should cause an IO");
        } catch (final IOException e) {
            // ok
        }
    }

    @Test
    public final void testConnect() throws IOException, InterruptedException {
        final PluggableConsoleInput input = new PluggableConsoleInput();

        input.disconnect();
        try (final PipedInputStream pis = new PipedInputStream();
             final PipedOutputStream pos = new PipedOutputStream();
             final PipedInputStream inner = new PipedInputStream(pos);
             final PipedOutputStream innerPos = new PipedOutputStream(pis);
             final PrintStream testIn = new PrintStream(innerPos, true, "UTF8");
             final PrintStream out = new PrintStream(pos)) {
            input.connect(pis, out);
            try {
                input.connect(pis, out);
                fail("Should not be able to connect already connected");
            } catch (final IOException e) {
                // ok
            }
            input.disconnect();

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        input.prompt("Test", 5000);
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }
            });
            th.start();

            while (!input.isPrompting()) {
                Thread.sleep(10);
            }

            input.connect(pis, out);
            testIn.println("tac");

            final Thread th2 = new Thread(new Runnable() {

                @Override
                public void run() {
                    try {
                        input.prompt("Test", 5000);
                        fail("Prompt should io");
                    } catch (final IOException e) {
                        // ok
                    }
                }
            });
            th2.start();
            while (!input.isPrompting()) {
                Thread.sleep(10);
            }

            input.close();


        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.socket.PluggableConsoleInput#getPrompt()}. */
    @Test
    public final void testGetPrompt() {
        final PluggableConsoleInput input = new PluggableConsoleInput();
        assertEquals("Default prompt invalid", "> ", input.getPrompt());
        input.setPrompt("test");
        assertEquals("Prompt setting failed", "test", input.getPrompt());
    }
}
